From 23133db4af5e6ae1fa636f4cd4430291744d39d9 Mon Sep 17 00:00:00 2001
From: Shugo Maeda <shugo@ruby-lang.org>
Date: Mon, 22 Nov 2010 16:12:47 +0900
Subject: [PATCH 50/51] refine creates only one module per class.

---
 eval.c                       |   27 ++++++++++++---------------
 test/ruby/test_refinement.rb |   25 +++++++++++++++++++++++--
 2 files changed, 35 insertions(+), 17 deletions(-)

diff --git a/eval.c b/eval.c
index 1190a99..f5d90c2 100644
--- a/eval.c
+++ b/eval.c
@@ -899,14 +899,12 @@ rb_overlay_module(NODE *cref, VALUE klass, VALUE module)
 }
 
 static int
-use_module_i(VALUE klass, VALUE modules, VALUE arg)
+use_module_i(VALUE klass, VALUE module, VALUE arg)
 {
     NODE *cref = (NODE *) arg;
     int i;
 
-    for (i = 0; i < RARRAY_LEN(modules); i++) {
-	rb_overlay_module(cref, klass, RARRAY_PTR(modules)[i]);
-    }
+    rb_overlay_module(cref, klass, module);
     return ST_CONTINUE;
 }
 
@@ -975,7 +973,7 @@ static VALUE
 rb_mod_refine(VALUE module, VALUE klass)
 {
     NODE *cref = rb_vm_cref();
-    VALUE mod = rb_module_new();
+    VALUE mod;
     ID id_overlayed_modules, id_refined_class;
     VALUE overlayed_modules, modules;
 
@@ -987,17 +985,16 @@ rb_mod_refine(VALUE module, VALUE klass)
 	rb_funcall(overlayed_modules, rb_intern("compare_by_identity"), 0);
 	rb_ivar_set(module, id_overlayed_modules, overlayed_modules);
     }
-    modules = rb_hash_aref(overlayed_modules, klass);
-    if (NIL_P(modules)) {
-	modules = rb_ary_new();
-	rb_hash_aset(overlayed_modules, klass, modules);
+    mod = rb_hash_aref(overlayed_modules, klass);
+    if (NIL_P(mod)) {
+	mod = rb_module_new();
+	CONST_ID(id_refined_class, "__refined_class__");
+	rb_ivar_set(mod, id_refined_class, klass);
+	rb_define_singleton_method(mod, "method_added",
+				   refinement_module_method_added, 1);
+	rb_overlay_module(cref, klass, mod);
+	rb_hash_aset(overlayed_modules, klass, mod);
     }
-    rb_ary_push(modules, mod);
-    CONST_ID(id_refined_class, "__refined_class__");
-    rb_ivar_set(mod, id_refined_class, klass);
-    rb_define_singleton_method(mod, "method_added",
-			       refinement_module_method_added, 1);
-    rb_overlay_module(cref, klass, mod);
     rb_mod_module_eval(0, NULL, mod);
     return mod;
 }
diff --git a/test/ruby/test_refinement.rb b/test/ruby/test_refinement.rb
index 8161449..c880643 100644
--- a/test/ruby/test_refinement.rb
+++ b/test/ruby/test_refinement.rb
@@ -180,10 +180,31 @@ class TestRefinement < Test::Unit::TestCase
     mod = nil
     result = nil
     m = Module.new {
-      result = refine Object do
+      result = refine(Object) {
         mod = self
-      end
+      }
     }
     assert_equal mod, result
   end
+
+  def test_refine_same_class_twice
+    result1 = nil
+    result2 = nil
+    result3 = nil
+    m = Module.new {
+      result1 = refine(Fixnum) {
+        def foo; return "foo" end
+      }
+      result2 = refine(Fixnum) {
+        def bar; return "bar" end
+      }
+      result3 = refine(String) {
+        def baz; return "baz" end
+      }
+    }
+    assert_equal("foo", m.module_eval { 1.foo })
+    assert_equal("bar", m.module_eval { 1.bar })
+    assert_equal(result1, result2)
+    assert_not_equal(result1, result3)
+  end
 end
-- 
1.7.0.4

